package aerobush;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpException;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.NameValuePair;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;

public class PixivGrabber
{
	final private HttpClient pixivClient;

	public PixivGrabber() {
		pixivClient = new HttpClient();

		GetMethod startPage = new GetMethod(
				"http://www.pixiv.net");
		startPage.setRequestHeader(
				"User-Agent",
				"Mozilla/4.0 (compatible; MSIE 6.0; Windows 2000)");
		startPage.setFollowRedirects(true);

		executeMethod(startPage);
	}

	public void authenticate(
			String username,
			String password ) {
		PostMethod loginPage = new PostMethod(
				"http://www.pixiv.net/index.php");
		loginPage.setRequestHeader(
				"User-Agent",
				"Mozilla/4.0 (compatible; MSIE 6.0; Windows 2000)");

		NameValuePair[] data = {
			new NameValuePair(
					"mode",
					"login"),
			new NameValuePair(
					"pixiv_id",
					username),
			new NameValuePair(
					"pass",
					password)
		};

		loginPage.setRequestBody(data);

		executeMethod(loginPage);
	}

	public void grabImages(
			String outputDirPath,
			String url ) {
		File outputDir = new File(
				outputDirPath);
		outputDir.mkdirs();

		GetMethod imagePage = new GetMethod(
				url);
		imagePage.setRequestHeader(
				"User-Agent",
				"Mozilla/4.0 (compatible; MSIE 6.0; Windows 2000)");
		imagePage.setFollowRedirects(true);
		executeMethod(imagePage);

		BufferedReader imagePageReader = new BufferedReader(
				new InputStreamReader(
						getStream(imagePage)));
		if (imagePageReader == null) return;

		System.out.println("Parsing " + url + ".");

		ArrayList<String> imageUrlList = new ArrayList<String>();
		String imagePageBody = readLine(imagePageReader);
		while (imagePageBody != null) {
			int index = imagePageBody.indexOf("illust_id=");
			while (index != -1) {
				index += 10;

				String imageUrl = "http://www.pixiv.net/member_illust.php?mode=big&illust_id=";
				while (imagePageBody.charAt(index) != '"') {
					imageUrl += imagePageBody.charAt(index);
					++index;
				}
				imageUrlList.add(imageUrl);

				index = imagePageBody.indexOf(
						"illust_id=",
						index);
			}
			imagePageBody = readLine(imagePageReader);
		}

		closeQuietly(imagePageReader);

		for (String imageUrl : imageUrlList) {
			grabImage(
					outputDirPath,
					imageUrl);
		}
	}

	private void grabImage(
			String outputDirPath,
			String url ) {
		GetMethod imagePage = new GetMethod(
				url);
		imagePage.setRequestHeader(
				"User-Agent",
				"Mozilla/4.0 (compatible; MSIE 6.0; Windows 2000)");
		url = url.replace(
				"big",
				"medium");
		imagePage.setRequestHeader(
				"Referer",
				url);
		imagePage.setFollowRedirects(true);
		executeMethod(imagePage);

		BufferedReader imagePageReader = new BufferedReader(
				new InputStreamReader(
						getStream(imagePage)));
		if (imagePageReader == null) return;

		boolean imageFound = false;
		String imageUrl = "";

		String imagePageBody = readLine(imagePageReader);
		while ((imagePageBody != null) && (!imageFound)) {
			int index = imagePageBody.indexOf("src=\"");
			while (index != -1) {
				index += 5;

				while (imagePageBody.charAt(index) != '"') {
					imageUrl += imagePageBody.charAt(index);
					++index;
				}
				imageFound = true;

				index = imagePageBody.indexOf(
						"src=\"",
						index);
			}
			imagePageBody = readLine(imagePageReader);
		}

		closeQuietly(imagePageReader);

		String filename = imageUrl.substring(imageUrl.lastIndexOf('/') + 1);
		downloadFile(
				url,
				imageUrl,
				new File(
						outputDirPath,
						filename));
	}

	private void downloadFile(
			String originUrl,
			String url,
			File file ) {
		GetMethod fileGrabber = new GetMethod(
				url);
		fileGrabber.setRequestHeader(
				"User-Agent",
				"Mozilla/4.0 (compatible; MSIE 6.0; Windows 2000)");
		fileGrabber.setRequestHeader(
				"Referer",
				originUrl);
		fileGrabber.setFollowRedirects(true);
		executeMethod(fileGrabber);

		System.out.println("Downloading file " + url + " from " + originUrl + " as " + file + ".");

		try {
			InputStream in = fileGrabber.getResponseBodyAsStream();
			FileOutputStream out = new FileOutputStream(
					file);
			byte[] buffer = new byte[1024];
			int count = -1;
			while ((count = in.read(buffer)) != -1) {
				out.write(
						buffer,
						0,
						count);
			}
			out.flush();
			out.close();
			in.close();
		}
		catch (IOException ioe) {
			System.out.println("Could not download file: " + url);
		}
	}

	private void executeMethod(
			HttpMethod method ) {
		try {
			pixivClient.executeMethod(method);
		}
		catch (HttpException e) {
			System.out.println("Http Exception.");
		}
		catch (IOException e) {
			System.out.println("IO Exception.");
		}
	}

	private InputStream getStream(
			GetMethod page ) {
		try {
			return page.getResponseBodyAsStream();
		}
		catch (IOException ioe) {
			return null;
		}
	}

	private String readLine(
			BufferedReader reader ) {
		try {
			return reader.readLine();
		}
		catch (Exception e) {
			return null;
		}
	}

	private void closeQuietly(
			BufferedReader reader ) {
		try {
			reader.close();
		}
		catch (IOException e) {}
	}

	public static void main(
			String[] args )
			throws Exception {
		if (args.length < 4) {
			System.out
					.println("Usage: java -jar PixivGrabber.java username password outputDir [pixiv urls separated by spaces]");
		}

		PixivGrabber imageGrabber = new PixivGrabber();

		imageGrabber.authenticate(
				args[0],
				args[1]);

		String outputDirPath = args[2];

		for (int i = 3; i < args.length; ++i) {
			imageGrabber.grabImages(
					outputDirPath,
					args[i]);
		}
	}
}